#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
#       Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
#       Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: 
#       Joan Sala Calero (joan.sala@bsc.es)

"""
Script: Generate a summary of the QAR production status
---
- Read QAR directory
- Generate bokeh table plot
"""

import os
import shutil
import pandas as pd
import configparser
from datetime import date
from math import pi

from bokeh.io import save, output_file
from bokeh.models import LinearColorMapper
from bokeh.plotting import figure
from bokeh.layouts import column

tests = [
	'file_format', 'standard_compliance', 
	'temporal_completeness', 'temporal_consistency',
	'spatial_completeness', 'spatial_consistency', 'valid_ranges'
]

"""
Generate a Bokeh table plot with tooltips
Each row is a QAR
Filter by keyword
"""
def table_plot(data, filt):

	# Filter data
	data=data[data.fname.str.contains('^{}'.format(filt))]
	
	# Count totals and drop column
	all_test = data.loc[data['test'] == 'all']
	completed = all_test.res.sum()
	available = all_test.res.count()
	data = data.loc[data['test'] != 'all']

	# Color mapper
	colors = ["#550b1d",  "#dddddd",  "#75968f"]
	mapper = LinearColorMapper(palette=colors, low=-1, high=1)

	yy = data.fname.unique()

	# length of the plot
	if len(yy) < 20: 
		ph = len(yy)*30
	elif len(yy) < 40: 
		ph = len(yy)*25
	else:
		ph = len(yy)*17

	p = figure(title="{} ({}) {} of {} done".format(
		       filt, date.today().strftime('%d %b'), completed, available),
	     	   x_range=tests, y_range=yy,
	           x_axis_location="above", plot_width=1000, plot_height=ph,
	           tools='', toolbar_location='below', 
	           tooltips=[('id', '@fname'),('test', '@test'),('msg', '@msg')])

	p.title.text_font_size = '9pt'
	p.toolbar.logo = None
	p.toolbar_location = None
	p.grid.grid_line_color = None
	p.axis.axis_line_color = None
	p.axis.major_tick_line_color = None
	p.axis.major_label_text_font_size = '7pt'
	p.axis.major_label_standoff = 0
	p.xaxis.major_label_orientation = pi / 3

	p.rect(x="test", y="fname", width=1, height=1,
	       source=data,
	       fill_color={'field': 'res', 'transform': mapper},
	       line_color=None)

	return p

"""
OK/NOK to integers
"""
def ok_nok(val):
	if val == 'ok':
		return 1
	else:
		return -1

"""
Get information from QAR/ini result
"""
def parse_ini(ff):

	# There is empty
	res = {}
	metadata = {}

	if os.path.exists(ff):
		ini = configparser.ConfigParser()
		ini.read(ff)
		res = {}
		metadata = {}

		for s in ini.sections():
			if 'result' in s:
				res[s.replace('_result', '')] = ok_nok(ini[s]['res'])	
				metadata[s.replace('_result', '')] = '{}: {}'.format(ini[s]['res'], ini[s]['msg'])

	return res, metadata

"""
Generate QAR report main
"""		    
def main():

	# Get all ini files
	script_dir = os.path.dirname(os.path.realpath(__file__))
	conf_dir = os.path.join(script_dir, 'conf')
	
	# Results dir
	res_dir = '/shared/QAR_dqc/results'
	os.chdir(res_dir)
	df = pd.DataFrame(columns=['fname', 'test', 'res'])
	
	for f in os.listdir(conf_dir):
		ini_res = os.path.join(res_dir, f)
		rs, mdata = parse_ini(ini_res)
		completed = 0

		# Data available
		if rs != {}:
			tt = 0
			for k,v in rs.items():			
				df=df.append({'fname':f.replace('.ini', ''), 'test': k, 'res': v, 'msg': mdata[k]}, 
					ignore_index=True)
				tt+=1

			# All checks passed
			completed = (len(tests) == tt)
		else:			
			for c in tests:		
				df=df.append({'fname':f.replace('.ini', ''), 'test': c, 'res': 0, 'msg': 'data unavailable'}, 
					ignore_index=True)

		# Add completed status
		df=df.append({'fname':f.replace('.ini', ''), 'test': 'all', 'res': completed, 'msg': ''}, 
						ignore_index=True)

	# Plot html table
	ps1=table_plot(df, 'seasonal-monthly-pressure-levels')
	ps2=table_plot(df, 'seasonal-monthly-single-levels')	
	ps3=table_plot(df, 'seasonal-postprocessed-pressure-levels')
	ps4=table_plot(df, 'seasonal-postprocessed-single-levels')
	
		
	pr1=table_plot(df, 'reanalysis-era5-pressure-levels-monthly-means')
	pr2=table_plot(df, 'reanalysis-era5-single-levels-monthly-means')

	# Save plot
	output_file(os.path.join(res_dir, 'qar_status.html'))
	save(column(ps1, ps2, ps3, ps4, pr1, pr2))


if __name__ == "__main__":
    main()