#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
#       Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
#       Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: 
#       Joan Sala Calero (joan.sala@bsc.es)

import os
import json
import subprocess
import eccodes as ecc

"""
Function to load GRIB metadata
"""
def load_grib_metadata(version=2):
	curr_dir = os.path.dirname(os.path.realpath(__file__))
	grib_file = os.path.join(curr_dir.replace('utils', 'grib/grib{}.json'.format(version)))
	with open(grib_file, 'r') as fp:
		return json.load(fp)

"""
Function used to get the maximum resolution within a grib file
Useful when opening dataset with xarray/cfgrib (key: numberOfDataPoints)
"""
def grib_get_max_resolution(grib_file):
	cmd = "grib_get -p numberOfDataPoints %s" %grib_file
	proc = subprocess.Popen(cmd, 
		shell=True,
	    stdout=subprocess.PIPE,
	    stderr=subprocess.STDOUT
	)
	
	# Get maximum resolution (if available)
	maxres = -1
	while True:
		line = proc.stdout.readline()
		try:
			maxres = max(maxres, int(line.strip()))
		except:
			# EOF
			if not line:
				break

	return maxres

"""
Class: Wrapper with utils around the GRIB1 and GRIB2 file formats
---
- Uses eccodes
"""
class GribFileCheck:

	def __init__(self, grib_file):        
		self.ecc_data = ecc.GribFile(grib_file)
		self.version = self.get_grib_version()

	"""
	Get all messages stored on a grib file
	"""
	def get_message_list(self):
		# This funtion returns all messages within a GRIB file
		msgs = []

		for i in range(len(self.ecc_data)):
			msgs.append(ecc.GribMessage(self.ecc_data))
		
		return msgs

	"""
	This functions loops over a GRIB2 file and checks for certain key,value pairs
	"""
	def check_mandatory_keys(self, check_keys):

		err_msgs = []
		msg = ecc.GribMessage(self.ecc_data)
		for ckey,cval in check_keys.items():  
			for i in range(len(self.ecc_data)):
				try:
					if int(msg[ckey]) != int(cval.strip()):
						err_msgs.append('GRIB Key \'{}\' has value \'{}\' instead of value \'{}\''.format(ckey, msg[ckey], cval))
						break # no need to proceed
				except ValueError:
					pass

		# All messages had the appropiate keys
		return err_msgs

	"""
	Get GRIB version from the first message. Returns integer
	"""
	def get_grib_version(self):

		first_message = self.ecc_data.next()
		keys = first_message.keys()

		if 'GRIBEditionNumber' in keys:
			return first_message['GRIBEditionNumber']

		# Not a valid GRIB file
		return None